@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2006 Sun Microsystems Inc.
+# Copyright 2006-2007 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -17,10 +17,10 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
-__id__        = "$Id: J2SE-access-bridge.py 2140 2007-03-07 20:04:06Z richb $"
-__version__   = "$Revision: 2140 $"
-__date__      = "$Date: 2007-03-07 15:04:06 -0500 (Wed, 07 Mar 2007) $"
-__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__id__        = "$Id: J2SE-access-bridge.py 2454 2007-06-04 17:38:57Z lmonsanto $"
+__version__   = "$Revision: 2454 $"
+__date__      = "$Date: 2007-06-04 13:38:57 -0400 (Mon, 04 Jun 2007) $"
+__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import atspi
@@ -48,7 +48,7 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
     """
     def __init__(self, script):
         speechgenerator.SpeechGenerator.__init__(self, script)
-        
+
     def _getSpeechForLabel(self, obj, already_focused):
         """Get the speech for a label.
 
@@ -65,9 +65,17 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
 
         obj.was_selected = False
         if obj.state.count(Accessibility.STATE_EXPANDED) != 0:
+            # Translators: this represents the state of a node in a tree.
+            # 'expanded' means the children are showing.
+            # 'collapsed' means the children are not showing.
+            #
             utterances.append(_("expanded"))
         elif obj.state.count(Accessibility.STATE_EXPANDED) == 0 and \
                 obj.state.count(Accessibility.STATE_EXPANDABLE) != 0:
+            # Translators: this represents the state of a node in a tree.
+            # 'expanded' means the children are showing.
+            # 'collapsed' means the children are not showing.
+            #
             utterances.append(_("collapsed"))
         elif obj.state.count(Accessibility.STATE_SELECTED) != 0:
             utterances.append(_("selected"))
@@ -91,8 +99,18 @@ class Script(default.Script):
         Arguments:
         - app: the application to create a script for.
         """
+
+        # Set the debug level for all the methods in this script.
+        #
+        self.debugLevel = debug.LEVEL_FINEST
+
         default.Script.__init__(self, app)
 
+    def _debug(self, msg):
+        """ Convenience method for printing debug messages
+        """
+        debug.println(self.debugLevel, "J2SE-access-bridge.py: "+msg)
+
     def getSpeechGenerator(self):
         """Returns the speech generator for this script.
         """
@@ -107,7 +125,7 @@ class Script(default.Script):
         Returns True if the event is of interest.
         """
         keysym = keyboardEvent.event_string
-        keyboardEvent.hw_code = keybindings._getKeycode(keysym)
+        keyboardEvent.hw_code = keybindings.getKeycode(keysym)
         return default.Script.consumesKeyboardEvent(self, keyboardEvent)
 
     def onFocus(self, event):
@@ -117,29 +135,32 @@ class Script(default.Script):
         - event: the Event
         """
 
+        self._debug("onFocus: %s '%s'" % \
+                    (event.source.role, event.source.name))
+
         role = event.source.role
         if role == rolenames.ROLE_LIST:
-	    selectedItem = None
-	    selection = event.source.selection
-	    if selection and selection.nSelectedChildren > 0:
-        	selectedItem = atspi.Accessible.makeAccessible(
-            				selection.getSelectedChild(0))
-	    elif event.source.childCount > 0:
-		selectedItem = event.source.child(0)
-	    if selectedItem:
-		orca.setLocusOfFocus(event, selectedItem)
-	    else:
-		# if impossible to get selection or list has 0 items, present list
-		orca.setLocusOfFocus(event, event.source)
-	elif role == rolenames.ROLE_LABEL:
-	    # In FileChooserDemo, when enter in a new folder, a focus event for 
-	    # the top combo box selected item (not SHOWING item) is received.
-	    # Should this check be more specific ?
-	    #
-	    if not event.source.state.count (Accessibility.STATE_SHOWING):
-		return
-	elif role == rolenames.ROLE_MENU:
-    	    # A JMenu has always selection.nSelectedChildren > 0
+            selectedItem = None
+            selection = event.source.selection
+            if selection and selection.nSelectedChildren > 0:
+                selectedItem = atspi.Accessible.makeAccessible(
+                                        selection.getSelectedChild(0))
+            elif event.source.childCount > 0:
+                selectedItem = event.source.child(0)
+            if selectedItem:
+                orca.setLocusOfFocus(event, selectedItem)
+            else:
+                # if impossible to get selection or list has 0 items, present list
+                orca.setLocusOfFocus(event, event.source)
+        elif role == rolenames.ROLE_LABEL:
+            # In FileChooserDemo, when enter in a new folder, a focus event for
+            # the top combo box selected item (not SHOWING item) is received.
+            # Should this check be more specific ?
+            #
+            if not event.source.state.count (Accessibility.STATE_SHOWING):
+                return
+        elif role == rolenames.ROLE_MENU:
+            # A JMenu has always selection.nSelectedChildren > 0
             orca.setLocusOfFocus(event, event.source)
         else:
             default.Script.onFocus(self, event)
@@ -152,6 +173,9 @@ class Script(default.Script):
         - event: the Event
         """
 
+        self._debug("onStateChanged: %s: %s '%s' (%d,%d)" % \
+                    (event.type, event.source.role, event.source.name, event.detail1, event.detail2))
+
         # This is a workaround for a java-access-bridge bug (Bug 355011)
         # where popup menu events are not sent to Orca.
         #
@@ -175,13 +199,13 @@ class Script(default.Script):
                             if item.state.count(Accessibility.STATE_ARMED):
                                 orca.setLocusOfFocus(event, item)
                                 return
-        
 
-	# In java applications the events are comming in other order:
-	# "focus:" event comes before "state-changed:focused" event
-	#
-	if (event.type == "object:state-changed:focused"):
-	    return
+
+        # In java applications the events are comming in other order:
+        # "focus:" event comes before "state-changed:focused" event
+        #
+        if (event.type == "object:state-changed:focused"):
+            return
 
         # Hand state changes when JTree labels become expanded
         # or collapsed.
@@ -189,18 +213,18 @@ class Script(default.Script):
         if ((event.source.role == rolenames.ROLE_LABEL) and \
             (event.type == "object:state-changed:expanded")):
             orca.visualAppearanceChanged(event, event.source)
-	    return
+            return
 
-	# Present a value change in case of an focused popup menu.
-	# Fix for Swing file chooser.
-	#
-	if event.type == "object:state-changed:visible" and \
-		event.source.role == rolenames.ROLE_POPUP_MENU and \
-		event.source.parent.state.count(Accessibility.STATE_FOCUSED):
-	    orca.setLocusOfFocus(event, event.source.parent)
-	    return
-	
-	default.Script.onStateChanged(self, event)
+        # Present a value change in case of an focused popup menu.
+        # Fix for Swing file chooser.
+        #
+        if event.type == "object:state-changed:visible" and \
+                event.source.role == rolenames.ROLE_POPUP_MENU and \
+                event.source.parent.state.count(Accessibility.STATE_FOCUSED):
+            orca.setLocusOfFocus(event, event.source.parent)
+            return
+
+        default.Script.onStateChanged(self, event)
 
     def onSelectionChanged(self, event):
         """Called when an object's selection changes.
@@ -209,8 +233,11 @@ class Script(default.Script):
         - event: the Event
         """
 
-	if not event.source.state.count (atspi.Accessibility.STATE_FOCUSED):
-	    return
+        self._debug("onSelectionChanged: %s '%s'" % \
+                    (event.source.role, event.source.name))
+
+        if not event.source.state.count (atspi.Accessibility.STATE_FOCUSED):
+            return
 
         if event.source.role == rolenames.ROLE_TABLE:
             return
@@ -253,11 +280,15 @@ class Script(default.Script):
     def onValueChanged(self, event):
         """Called whenever an object's value changes.  Currently, the
         value changes for non-focused objects are ignored.
-        
+
         Arguments:
         - event: the Event
         """
-        
+
+        self._debug("onValueChanged: %s '%s' value=%d" % \
+                    (event.source.role, event.source.name,
+                     event.source.value.currentValue))
+
         # We'll let state-changed:checked event to be used to
         # manage check boxes in java application
         #
@@ -265,6 +296,8 @@ class Script(default.Script):
                event.type == "object:property-change:accessible-value":
             return
 
+        default.Script.onValueChanged(self, event)
+
 
     def visualAppearanceChanged(self, event, obj):
         """Called when the visual appearance of an object changes.  This
@@ -279,6 +312,9 @@ class Script(default.Script):
         - obj: the Accessible whose visual appearance changed.
         """
 
+        self._debug("visualAppearanceChanged: %s '%s', locusOfFocus=%s" % \
+                    (obj.role, obj.name, orca_state.locusOfFocus.role))
+
         # Present to Braille and Speech an object that is the same
         # with the last focused object, only if it is a label.
         # This case is for LISTs which contain labels as items
@@ -290,5 +326,82 @@ class Script(default.Script):
                 speech.speakUtterances(self.speechGenerator.getSpeech(orca_state.locusOfFocus, True))
                 return
 
+        if obj.role == rolenames.ROLE_SPIN_BOX:
+            # Check for the spinbox text object being null. There appears
+            # to be a bug where the text object for some text fields
+            # is null. 
+            if orca_state.locusOfFocus.role == rolenames.ROLE_TEXT and \
+               orca_state.locusOfFocus.text == None:
+                # Set the locusOfFocus to the spinbox itself.
+                orca.setLocusOfFocus(event, obj)
+
         default.Script.visualAppearanceChanged(self, event, obj)
 
+    def onCaretMoved(self, event):
+        """Called whenever the caret moves.
+
+        Arguments:
+        - event: the Event
+        """
+
+        self._debug("onCaretMoved: %s '%s'" % \
+                    (event.source.role, event.source.name))
+
+        # Check for bogus events containing null text objects.
+        if event.source.text == None:
+            self._debug("bogus event: text object is null. Discarding event")
+            return;
+
+        default.Script.onCaretMoved(self, event)
+
+    def onTextInserted(self, event):
+        """Called whenever the text is inserted.
+
+        Arguments:
+        - event: the Event
+        """
+
+        self._debug("onTextInserted: %s '%s'" % \
+                    (event.source.role, event.source.name))
+
+        # Check for bogus events containing null text objects.
+        if event.source.text == None:
+            self._debug("bogus event: text object is null. Discarding event")
+            return;
+
+        default.Script.onTextInserted(self, event)        
+
+    def getDisplayedLabel(self, object):
+        """If there is an object labelling the given object, return the
+        text being displayed for the object labelling this object.
+        Otherwise, return None.
+
+        Argument:
+        - object: the object in question
+
+        Returns the string of the object labelling this object, or None
+        if there is nothing of interest here.
+        """
+        label = default.Script.getDisplayedLabel(self, object)
+        if not label and \
+           (object.role == rolenames.ROLE_TEXT or \
+            object.role == rolenames.ROLE_COMBO_BOX):
+            # This is a workaround for a common Java accessibility
+            # bug where a label next to a component is used to
+            # label the component. This relation is only obvious to
+            # a sighted person. The LABEL_FOR property is used to
+            # indicate the label refers to the component next to it.
+            # The workaround is to assume the label refers to the
+            # component if the label is the previous component and
+            # ends with a colon. We only do this for text and combo
+            # boxes since they are the most common Java component with
+            # this bug.
+            if object.index > 0:
+                prevSibling = object.parent.child(object.index-1)
+                if prevSibling and prevSibling.name and \
+                   prevSibling.role == rolenames.ROLE_LABEL:
+                    labelFor = prevSibling.name.rstrip(" ")
+                    if (labelFor.endswith(":")):
+                        return labelFor.rstrip(":")
+
+        return label
