@@ -25,9 +25,9 @@ operations on the Accessible object, such as invoking default actions or
 moving the text caret.
 """
 
-__id__        = "$Id: braille.py 2223 2007-03-28 17:06:38Z wwalker $"
-__version__   = "$Revision: 2223 $"
-__date__      = "$Date: 2007-03-28 13:06:38 -0400 (Wed, 28 Mar 2007) $"
+__id__        = "$Id: braille.py 2427 2007-05-27 19:23:28Z wwalker $"
+__version__   = "$Revision: 2427 $"
+__date__      = "$Date: 2007-05-27 15:23:28 -0400 (Sun, 27 May 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -35,7 +35,6 @@ import logging
 log = logging.getLogger("braille")
 
 import signal
-import threading
 
 import atspi
 
@@ -44,9 +43,12 @@ import atspi
 #
 try:
     import brlapi
+    import gobject
+
     brlAPI = None
     useBrlAPIBindings = True
     brlAPIRunning = False
+    brlAPISourceId = 0
 except:
     import brl
     useBrlAPIBindings = False
@@ -283,9 +285,9 @@ class Region:
 
         self.cursorOffset = cursorOffset
 
-    def processCursorKey(self, offset):
-        """Processes a cursor key press on this Component.  The offset is
-        0-based, where 0 represents the leftmost character of string
+    def processRoutingKey(self, offset):
+        """Processes a cursor routing key press on this Component.  The offset
+        is 0-based, where 0 represents the leftmost character of string
         associated with this region.  Note that the zeroeth character may have
         been scrolled off the display."""
         pass
@@ -309,9 +311,9 @@ class Component(Region):
         Region.__init__(self, string, cursorOffset)
         self.accessible = accessible
 
-    def processCursorKey(self, offset):
-        """Processes a cursor key press on this Component.  The offset is
-        0-based, where 0 represents the leftmost character of string
+    def processRoutingKey(self, offset):
+        """Processes a cursor routing key press on this Component.  The offset
+        is 0-based, where 0 represents the leftmost character of string
         associated with this region.  Note that the zeroeth character may have
         been scrolled off the display."""
 
@@ -326,7 +328,7 @@ class Component(Region):
             # routing key.]]]
             #
             debug.println(debug.LEVEL_FINEST,
-                          "braille.Component.processCursorKey: no action")
+                          "braille.Component.processRoutingKey: no action")
             try:
                 eventsynthesizer.clickObject(self.accessible, 1)
             except:
@@ -340,7 +342,7 @@ class Text(Region):
     [[[TODO: WDW - need to add in text selection capabilities.  Logged
     as bugzilla bug 319754.]]]"""
 
-    def __init__(self, accessible, label=None):
+    def __init__(self, accessible, label=None, eol=""):
         """Creates a new Text region.
 
         Arguments:
@@ -367,7 +369,11 @@ class Text(Region):
                     0,
                     atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
 
-        cursorOffset = self.caretOffset - self.lineOffset
+        cursorOffset = min(self.caretOffset - self.lineOffset, len(string))
+
+        self._maxCaretOffset = self.lineOffset + len(string.decode("UTF-8"))
+
+        string = string + eol
 
         self.label = label
         if self.label:
@@ -385,7 +391,7 @@ class Text(Region):
 
         [string, caretOffset, lineOffset] = \
                  orca_state.activeScript.getTextLineAtCaret(self.accessible)
-        cursorOffset = caretOffset - lineOffset
+        cursorOffset = min(caretOffset - lineOffset, len(string))
         if self.label:
             cursorOffset += len(self.label.decode("UTF-8")) + 1
 
@@ -398,18 +404,18 @@ class Text(Region):
 
         return True
 
-    def processCursorKey(self, offset):
-        """Processes a cursor key press on this Component.  The offset is
-        0-based, where 0 represents the leftmost character of text associated
-        with this region.  Note that the zeroeth character may have been
-        scrolled off the display."""
+    def processRoutingKey(self, offset):
+        """Processes a cursor routing key press on this Component.  The offset
+        is 0-based, where 0 represents the leftmost character of text
+        associated with this region.  Note that the zeroeth character may have
+        been scrolled off the display."""
 
         if self.label:
             offset = offset - len(self.label.decode("UTF-8")) - 1
             if offset < 0:
                 return
 
-        newCaretOffset = self.lineOffset + offset
+        newCaretOffset = min(self.lineOffset + offset, self._maxCaretOffset)
         self.accessible.text.setCaretOffset(newCaretOffset)
 
 class ReviewComponent(Component):
@@ -451,11 +457,11 @@ class ReviewText(Region):
         self.lineOffset = lineOffset
         self.zone = zone
 
-    def processCursorKey(self, offset):
-        """Processes a cursor key press on this Component.  The offset is
-        0-based, where 0 represents the leftmost character of text associated
-        with this region.  Note that the zeroeth character may have been
-        scrolled off the display."""
+    def processRoutingKey(self, offset):
+        """Processes a cursor routing key press on this Component.  The offset
+        is 0-based, where 0 represents the leftmost character of text
+        associated with this region.  Note that the zeroeth character may have
+        been scrolled off the display."""
 
         newCaretOffset = self.lineOffset + offset
         self.accessible.text.setCaretOffset(newCaretOffset)
@@ -522,16 +528,20 @@ class Line:
             else:
                 pos = len(string.decode("UTF-8"))
 
-        return [region, offset - pos]
+        if offset >= len(string.decode("UTF-8")):
+            return [None, -1]
+        else:
+            return [region, offset - pos]
 
-    def processCursorKey(self, offset):
-        """Processes a cursor key press on this Component.  The offset is
-        0-based, where 0 represents the leftmost character of string
+    def processRoutingKey(self, offset):
+        """Processes a cursor routing key press on this Component.  The offset
+        is 0-based, where 0 represents the leftmost character of string
         associated with this line.  Note that the zeroeth character may have
         been scrolled off the display."""
 
         [region, regionOffset] = self.getRegionAtOffset(offset)
-        region.processCursorKey(regionOffset)
+        if region:
+            region.processRoutingKey(regionOffset)
 
 def getRegionAtCell(cell):
     """Given a 1-based cell offset, return the braille region
@@ -924,7 +934,7 @@ def _processBrailleEvent(command):
         if len(_lines) > 0:
             cursor = (command - 0x100) + _viewport[0]
             lineNum = _viewport[1]
-            _lines[lineNum].processCursorKey(cursor)
+            _lines[lineNum].processRoutingKey(cursor)
             consumed = True
 
     if settings.timeoutCallback and (settings.timeoutTime > 0):
@@ -932,9 +942,9 @@ def _processBrailleEvent(command):
 
     return consumed
 
-def _brlAPIKeyReader():
+def _brlAPIKeyReader(source, condition):
         """Method to read a key from the BrlAPI bindings.  This is a
-        gidle handler.
+        gobject IO watch handler.
         """
         key = brlAPI.readKey(False)
         if key:
@@ -1040,10 +1050,11 @@ def init(callback=None, tty=7):
 
     if useBrlAPIBindings:
         try:
-            import gobject
-            gobject.threads_init()
             global brlAPI
             global brlAPIRunning
+            global brlAPISourceId
+
+            gobject.threads_init()
             brlAPI = brlapi.Connection()
 
             try:
@@ -1051,7 +1062,6 @@ def init(callback=None, tty=7):
                 windowPath = os.environ["WINDOWPATH"]
                 brlAPI.enterTtyModeWithPath()
                 brlAPIRunning = True
-                gobject.idle_add(_brlAPIKeyReader)
                 debug.println(\
                     debug.LEVEL_CONFIGURATION,
                     "Braille module has been initialized using WINDOWPATH=" \
@@ -1059,10 +1069,12 @@ def init(callback=None, tty=7):
             except:
                 brlAPI.enterTtyMode(tty)
                 brlAPIRunning = True
-                gobject.idle_add(_brlAPIKeyReader)
                 debug.println(\
                     debug.LEVEL_CONFIGURATION,
                     "Braille module has been initialized using tty=%d" % tty)
+            brlAPISourceId = gobject.io_add_watch(brlAPI.fileDescriptor,
+                                                  gobject.IO_IN,
+                                                  _brlAPIKeyReader)
         except:
             debug.printException(debug.LEVEL_FINEST)
             return False
@@ -1108,9 +1120,13 @@ def shutdown():
         return False
 
     global brlAPIRunning
+    global brlAPISourceId
+
     if useBrlAPIBindings:
         if brlAPIRunning:
             brlAPIRunning = False
+            gobject.source_remove(brlAPISourceId)
+            brlAPISourceId = 0
             brlAPI.leaveTtyMode()
     else:
         brl.shutdown()
