@@ -19,9 +19,9 @@
 
 """Provides the default implementation for flat review for Orca."""
 
-__id__        = "$Id: flat_review.py,v 1.47 2006/10/25 18:23:44 wwalker Exp $"
-__version__   = "$Revision: 1.47 $"
-__date__      = "$Date: 2006/10/25 18:23:44 $"
+__id__        = "$Id: flat_review.py 2465 2007-06-06 19:47:21Z lmonsanto $"
+__version__   = "$Revision: 2465 $"
+__date__      = "$Date: 2007-06-06 15:47:21 -0400 (Wed, 06 Jun 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -34,7 +34,6 @@ import debug
 import eventsynthesizer
 import orca_state
 import rolenames
-import util
 
 # [[[WDW - HACK Regular expression to split strings on whitespace
 # boundaries, which is what we'll use for word dividers instead of
@@ -319,9 +318,12 @@ class Line:
             self.brailleRegions = []
             brailleOffset = 0
             for zone in self.zones:
-                if (zone.accessible.role == rolenames.ROLE_TEXT) \
-                   or (zone.accessible.role == rolenames.ROLE_PASSWORD_TEXT) \
-                   or (zone.accessible.role == rolenames.ROLE_TERMINAL):
+                # The 'isinstance(zone, TextZone)' test is a sanity check
+                # to handle problems with Java text. See Bug 435553.
+                if isinstance(zone, TextZone) and \
+                   ((zone.accessible.role == rolenames.ROLE_TEXT) \
+                    or (zone.accessible.role == rolenames.ROLE_PASSWORD_TEXT) \
+                    or (zone.accessible.role == rolenames.ROLE_TERMINAL)):
                     region = braille.ReviewText(zone.accessible,
                                                 zone.string,
                                                 zone.startOffset,
@@ -399,8 +401,10 @@ class Context:
         currentWordIndex = 0
         currentCharIndex = 0
 
-        if orca_state.locusOfFocus.role == rolenames.ROLE_TABLE_CELL:
-            searchZone = util.getRealActiveDescendant(orca_state.locusOfFocus)
+        if orca_state.locusOfFocus and \
+           orca_state.locusOfFocus.role == rolenames.ROLE_TABLE_CELL:
+            searchZone = orca_state.activeScript.getRealActiveDescendant(\
+                                                   orca_state.locusOfFocus)
         else:
             searchZone = orca_state.locusOfFocus
 
@@ -434,16 +438,22 @@ class Context:
             zoneIndex   = currentZoneIndex
             caretOffset = zone.accessible.text.caretOffset
             foundZoneWithCaret = False
+            checkForEOF = False
             while lineIndex < len(self.lines):
                 line = self.lines[lineIndex]
                 while zoneIndex < len(line.zones):
                     zone = line.zones[zoneIndex]
                     if zone.accessible == accessible:
-                        if (caretOffset >= zone.startOffset) \
-                               and (caretOffset \
+                        if (caretOffset >= zone.startOffset):
+                            if (caretOffset \
                                     < (zone.startOffset + zone.length)):
-                            foundZoneWithCaret = True
-                            break
+                                foundZoneWithCaret = True
+                                break
+                            elif (caretOffset \
+                                    == (zone.startOffset + zone.length)):
+                                checkForEOF = True
+                                lineToCheck = lineIndex
+                                zoneToCheck = zoneIndex
                     zoneIndex += 1
                 if foundZoneWithCaret:
                     currentLineIndex = lineIndex
@@ -463,6 +473,16 @@ class Context:
                 else:
                     zoneIndex = 0
                     lineIndex += 1
+            atEOF = not foundZoneWithCaret and checkForEOF
+            if atEOF:
+                line = self.lines[lineToCheck]
+                zone = line.zones[zoneToCheck]
+                currentLineIndex = lineToCheck
+                currentZoneIndex = zoneToCheck
+                if caretOffset:
+                    currentWordIndex = len(zone.words) - 1
+                    currentCharIndex = \
+                          zone.words[currentWordIndex].length - 1
 
         self.lineIndex = currentLineIndex
         self.zoneIndex = currentZoneIndex
@@ -555,7 +575,7 @@ class Context:
         #print "LOOKING AT '%s'" % unicodeString
         for i in range(0, len(unicodeString) + 1):
             if (i != len(unicodeString)) \
-               and (unicodeString[i] != util.EMBEDDED_OBJECT_CHARACTER):
+               and (unicodeString[i] != orca_state.activeScript.EMBEDDED_OBJECT_CHARACTER):
                 substringEndOffset += 1
             elif (substringEndOffset == substringStartOffset):
                 substringStartOffset += 1
@@ -649,7 +669,7 @@ class Context:
             #      % (offset, startOffset, endOffset, string)
             #if startOffset > offset:
             #    embedded = text.getText(offset, offset + 1).decode("UTF-8")
-            #    if embedded[0] == util.EMBEDDED_OBJECT_CHARACTER:
+            #    if embedded[0] == orca_state.activeScript.EMBEDDED_OBJECT_CHARACTER:
             #        offset = startOffset
 
             debug.println(debug.LEVEL_FINEST,
@@ -766,10 +786,18 @@ class Context:
         # We really want the accessible text information.  But, if we have
         # an image, and it has a description, we can fall back on it.
         #
-        if (len(zones) == 0) \
-               and accessible.image \
-               and accessible.image.imageDescription \
-               and len(accessible.image.imageDescription):
+        if (len(zones) == 0) and accessible.image:
+            # Check for accessible.name, if it exists and has len > 0, use it
+            # Otherwise, do the same for accessible.description
+            # Otherwise, do the same for accessible.image.description
+            imageName = ""
+            if accessible.name and len(accessible.name):
+                imageName = accessible.name
+            elif accessible.description and len(accessible.description):
+                imageName = accessible.description
+            elif accessible.image.imageDescription and \
+                     len(accessible.image.imageDescription):
+                imageName = accessible.image.imageDescription
 
             [x, y] = accessible.image.getImagePosition(0)
             [width, height] = accessible.image.getImageSize()
@@ -785,7 +813,7 @@ class Context:
 
                 if (clipping[2] != 0) or (clipping[3] != 0):
                     zones.append(Zone(accessible,
-                                      accessible.image.imageDescription,
+                                      imageName,
                                       clipping[0],
                                       clipping[1],
                                       clipping[2],
@@ -803,7 +831,10 @@ class Context:
         # accessible.
         #
         if (accessible.role != rolenames.ROLE_COMBO_BOX) \
+            and (accessible.role != rolenames.ROLE_EMBEDDED) \
+            and (accessible.role != rolenames.ROLE_LABEL) \
             and (accessible.role != rolenames.ROLE_MENU) \
+            and (accessible.role != rolenames.ROLE_PAGE_TAB) \
             and accessible.childCount > 0:
             pass
         elif len(zones) == 0:
@@ -938,12 +969,26 @@ class Context:
 
         if not root:
             return []
+        
+        zones = []
 
         # If we're at a leaf node, then we've got a good one on our hands.
         #
         if root.childCount <= 0:
             return self.getZonesFromAccessible(root, root.extents)
 
+        # Handle non-leaf Java JTree nodes. If the node is collapsed,
+        # treat it as a leaf node. If it's expanded, add it to the
+        # Zones list.
+        #
+        if root.state.count(atspi.Accessibility.STATE_EXPANDABLE):
+            if root.state.count(atspi.Accessibility.STATE_COLLAPSED):
+                return self.getZonesFromAccessible(root, root.extents)
+            elif root.state.count(atspi.Accessibility.STATE_EXPANDED):
+                treenode = self.getZonesFromAccessible(root, root.extents)
+                if treenode:
+                    zones.extend(treenode)
+
         # We'll stop at various objects because, while they do have
         # children, we logically think of them as one region on the
         # screen.  [[[TODO: WDW - HACK stopping at menu bars for now
@@ -953,6 +998,7 @@ class Context:
         #
         if (root.parent and (root.parent.role == rolenames.ROLE_MENU_BAR)) \
            or (root.role == rolenames.ROLE_COMBO_BOX) \
+           or (root.role == rolenames.ROLE_EMBEDDED) \
            or (root.role == rolenames.ROLE_TEXT):
             return self.getZonesFromAccessible(root, root.extents)
 
@@ -963,7 +1009,6 @@ class Context:
         # parents, especially those that implement accessible text.  Logged
         # as bugzilla bug 319773.]]]
         #
-        zones = []
         if root.role == rolenames.ROLE_PAGE_TAB:
             zones.extend(self.getZonesFromAccessible(root, root.extents))
 
@@ -989,7 +1034,7 @@ class Context:
                     debug.println(debug.LEVEL_WARNING,
                                   "flat_review.getShowingZones: " +
                                   "WARNING CHILD.PARENT != PARENT!!!")
-                elif child.state.count(atspi.Accessibility.STATE_SHOWING):
+                elif self.script.pursueForFlatReview(child):
                     zones.extend(self.getShowingZones(child))
 
         return zones
