@@ -19,9 +19,9 @@
 
 """The main module for the Orca screen reader."""
 
-__id__        = "$Id: orca.py 2067 2007-02-28 00:10:12Z wwalker $"
-__version__   = "$Revision: 2067 $"
-__date__      = "$Date: 2007-02-27 19:10:12 -0500 (Tue, 27 Feb 2007) $"
+__id__        = "$Id: orca.py 2468 2007-06-12 00:18:34Z wwalker $"
+__version__   = "$Revision: 2468 $"
+__date__      = "$Date: 2007-06-11 20:18:34 -0400 (Mon, 11 Jun 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -63,9 +63,6 @@ import rolenames
 import settings
 import speech
 
-import threading
-import util
-
 from input_event import BrailleEvent
 from input_event import KeyboardEvent
 from input_event import MouseButtonEvent
@@ -73,6 +70,14 @@ from input_event import InputEventHandler
 
 from orca_i18n import _           # for gettext support
 
+if settings.debugMemoryUsage:
+    import gc
+    gc.set_debug(gc.DEBUG_UNCOLLECTABLE
+                 | gc.DEBUG_COLLECTABLE
+                 | gc.DEBUG_INSTANCES
+                 | gc.DEBUG_OBJECTS
+                 | gc.DEBUG_SAVEALL)
+
 # The user-settings module (see loadUserSettings).
 #
 _userSettings = None
@@ -228,7 +233,7 @@ def _onChildrenChanged(e):
         #
         try:
             if registry.desktop.childCount == 0:
-                speech.speak(_("User logged out - shutting down."))
+                speech.speak(_("Goodbye."))
                 shutdown()
                 return
         except: # could be a CORBA.COMM_FAILURE
@@ -356,15 +361,6 @@ def toggleKeystrokeRecording(script=None, inputEvent=None):
 #                                                                      #
 ########################################################################
 
-def cycleDebugLevel(script=None, inputEvent=None):
-    """Cycles the debug level at run time.
-
-    Arguments:
-    - inputEvent: the InputEvent instance that caused this to be called.
-
-    Returns True indicating the event should be consumed.
-    """
-
 def cycleDebugLevel(script=None, inputEvent=None):
     global _debugLevel
 
@@ -392,13 +388,13 @@ def cycleDebugLevel(script=None, inputEvent=None):
     debug.debugLevel = level
 
     if level == debug.LEVEL_ALL:
-        speech.speak(_("Debug level all."))
+        speech.speak("Debug level all.")
     elif level == debug.LEVEL_FINEST:
-        speech.speak(_("Debug level finest."))
+        speech.speak("Debug level finest.")
     elif level == debug.LEVEL_FINER:
-        speech.speak(_("Debug level finer."))
+        speech.speak("Debug level finer.")
     elif level == debug.LEVEL_FINE:
-        speech.speak(("Debug level fine."))
+        speech.speak("Debug level fine.")
     elif level == debug.LEVEL_CONFIGURATION:
         speech.speak("Debug level configuration.")
     elif level == debug.LEVEL_INFO:
@@ -412,28 +408,6 @@ def cycleDebugLevel(script=None, inputEvent=None):
 
     return True
 
-def printApps(script=None, inputEvent=None):
-    """Prints a list of all applications to stdout."""
-    util.printApps()
-    return True
-
-def printActiveApp(script=None, inputEvent=None):
-    """Prints the currently active application."""
-    util.printActiveApp()
-    return True
-
-def printAncestry(script=None, inputEvent=None):
-    """Prints the ancestry for the current locusOfFocus"""
-    util.printAncestry(orca_state.locusOfFocus)
-    return True
-
-def printHierarchy(script=None, inputEvent=None):
-    """Prints the application for the current locusOfFocus"""
-    if orca_state.locusOfFocus:
-        util.printHierarchy(orca_state.locusOfFocus.app,
-                            orca_state.locusOfFocus)
-    return True
-
 ########################################################################
 #                                                                      #
 # METHODS FOR PRE-PROCESSING AND MASSAGING KEYBOARD EVENTS.            #
@@ -466,7 +440,9 @@ def _isPrintableKey(event_string):
     if event_string == "space":
         reply = True
     else:
-        reply = event_string in string.printable
+        unicodeString = event_string.decode("UTF-8")
+        reply = (len(unicodeString) == 1) \
+                and (unicodeString.isalnum() or unicodeString.isspace())
     debug.println(debug.LEVEL_FINEST,
                   "orca._echoPrintableKey: returning: %s" % reply)
     return reply
@@ -597,22 +573,22 @@ def _keyEcho(event):
                 return
 
             modifiers = event.modifiers
-            if event_string == _("Caps_Lock"):
+            if event_string == "Caps_Lock":
                 if modifiers & (1 << atspi.Accessibility.MODIFIER_SHIFTLOCK):
-                    lockState = _(" off")
+                    lockState = " " + _("off")
                 else:
-                    lockState = _(" on")
+                    lockState = " " + _("on")
 
-            elif event_string == _("Num_Lock"):
+            elif event_string == "Num_Lock":
                 # [[[TODO: richb - we are not getting a correct modifier
                 # state value returned when Num Lock is turned off.
                 # Commenting out the speaking of the bogus on/off state
                 # until this can be fixed.]]]
                 #
                 #if modifiers & (1 << atspi.Accessibility.MODIFIER_NUMLOCK):
-                #    event_string += _(" off")
+                #    event_string += " " + _("off")
                 #else:
-                #    event_string += _(" on")
+                #    event_string += " " + _("on")
                 pass
 
         elif _isFunctionKey(event_string):
@@ -658,6 +634,10 @@ def _processKeyCaptured(event):
         elif event.event_string == "Escape":
             orca_state.capturingKeys = False
         else:
+            # Translators: this is a spoken prompt letting the user know
+            # Orca has recorded a new key combination (e.g., Alt+Ctrl+g)
+            # based upon their input.
+            #
             speech.speak(_("Key captured: %s. Press enter to confirm.") \
                          % str(event.event_string))
             orca_state.lastCapturedKey = event
@@ -697,10 +677,39 @@ def _processKeyboardEvent(event):
 
     keyboardEvent = KeyboardEvent(event)
 
-    # See if this is one of our special Orca modifiers (e.g., "Insert")
+    # See if this is one of our special Orca modifier keys.
     #
-    orcaModifierKeys = settings.orcaModifierKeys
-    isOrcaModifier = orcaModifierKeys.count(keyboardEvent.event_string) > 0
+    # [[[TODO: WDW - Note that just looking at the keycode should
+    # suffice, but there is a "feature" in the Java Access Bridge
+    # where it chooses to emit Java platform-independent keycodes
+    # instead of the keycodes for the base platform:
+    #
+    # http://bugzilla.gnome.org/show_bug.cgi?id=106004
+    # http://bugzilla.gnome.org/show_bug.cgi?id=318615
+    #
+    # So...we need to workaround this problem.
+    #
+    # If you make the following expression True we will get a positive
+    # match for all keysyms associated with a given keysym specified
+    # as an Orca modifier key.
+    #
+    # For example, assume the Orca modifier is set to \ for some
+    # reason.  The key that has \ on it produces \ without the Shift
+    # key and | with the Shift key.  If the following expression is
+    # True, both the \ and | will be viewed as the Orca modifier.  If
+    # the following expression is False, only the \ will be viewed as
+    # the Orca modifier (i.e., Shift+\ will still function as the |
+    # character).  In general, I think we want to avoid sucking in all
+    # possible keysyms because it can have unexpected results.]]]
+    #
+    if False:
+        allPossibleKeysyms = []
+        for keysym in settings.orcaModifierKeys:
+            allPossibleKeysyms.extend(keybindings.getAllKeysyms(keysym))
+    else:
+        allPossibleKeysyms = settings.orcaModifierKeys
+
+    isOrcaModifier = allPossibleKeysyms.count(keyboardEvent.event_string) > 0
 
     if event.type == atspi.Accessibility.KEY_PRESSED_EVENT:
         # Key presses always interrupt speech.
@@ -723,7 +732,9 @@ def _processKeyboardEvent(event):
         _orcaModifierPressed = False
 
     if _orcaModifierPressed:
-        keyboardEvent.modifiers |= (1 << settings.MODIFIER_ORCA)
+        keyboardEvent.modifiers = keyboardEvent.modifiers \
+                                  | (1 << settings.MODIFIER_ORCA)
+
 
     # Orca gets first stab at the event.  Then, the presenter gets
     # a shot. [[[TODO: WDW - might want to let the presenter try first?
@@ -742,10 +753,12 @@ def _processKeyboardEvent(event):
             if (not consumed) and settings.learnModeEnabled:
                 if keyboardEvent.type \
                     == atspi.Accessibility.KEY_PRESSED_EVENT:
-                    clickCount = util.getClickCount(orca_state.lastInputEvent,
+                    clickCount = orca_state.activeScript.getClickCount(\
+                                                orca_state.lastInputEvent,
                                                 keyboardEvent)
                     if clickCount == 2:
-                        util.phoneticSpellCurrentItem(keyboardEvent.event_string)
+                        orca_state.activeScript.phoneticSpellCurrentItem(\
+                            keyboardEvent.event_string)
                     else:
                         # Check to see if there are localized words to be
                         # spoken for this key event.
@@ -760,6 +773,14 @@ def _processKeyboardEvent(event):
 
     orca_state.lastInputEvent = keyboardEvent
 
+    # If this is a key event for a non-modifier key, save a handle to it.
+    # This is needed to help determine user actions when a multi-key chord
+    # has been pressed, and we might get the key events in different orders.
+    # See comment #15 of bug #435201 for more details.
+    #
+    if not _isModifierKey(keyboardEvent.event_string):
+        orca_state.lastNonModifierKeyEvent = keyboardEvent
+
     return consumed or isOrcaModifier
 
 ########################################################################
@@ -813,8 +834,14 @@ def _toggleSilenceSpeech(script=None, inputEvent=None):
     speech.stop()
     if settings.silenceSpeech:
         settings.silenceSpeech = False
+        # Translators: this is a spoken prompt letting the user know
+        # that speech synthesis has been turned back on.
+        #
         speech.speak(_("Speech enabled."))
     else:
+        # Translators: this is a spoken prompt letting the user know
+        # that speech synthesis has been temporarily turned off.
+        #
         speech.speak(_("Speech disabled."))
         settings.silenceSpeech = True
     return True
@@ -863,6 +890,10 @@ def loadUserSettings(script=None, inputEvent=None):
         try:
             speech.init()
             if reloaded:
+                # Translators: there is a keystroke to reload the user
+                # preferences.  This is a spoken prompt to let the user
+                # know when the preferences has been reloaded.
+                #
                 speech.speak(_("Orca user settings reloaded."))
             debug.println(debug.LEVEL_CONFIGURATION,
                           "Speech module has been initialized.")
@@ -895,9 +926,30 @@ def loadUserSettings(script=None, inputEvent=None):
         debug.println(debug.LEVEL_CONFIGURATION,
                       "Magnification module has NOT been initialized.")
 
+    # We don't want the Caps_Lock modifier to act as a locking
+    # modifier if it used as the Orca modifier key.  In addition, if
+    # the KP_Insert key is used as the Orca modifier key, we want to
+    # make sure we clear any other keysyms that might be in use on
+    # that key since we won't be able to detect them as being the Orca
+    # modifier key.  For example, KP_Insert produces "KP_Insert" when
+    # pressed by itself, but Shift+KP_Insert produces "0".
+    #
+    # The original values are saved/reset in the orca shell script.
+    #
+    # [[[TODO: WDW - we probably should just to a 'xmodmap -e "%s = %s"'
+    # for all of the orcaModifierKeys, but saving/restoring the values
+    # becomes a little more difficult.  If we could assume a writeable
+    # filesystem (we cannot), we could do a 'xmodmap -pke > /tmp/foo'
+    # to save the keymap and a 'xmodmap /tmp/foo' to restore it.
+    # For now, we'll just look at the Orca modifier keys we support
+    # (Caps Lock, KP_Insert, and Insert).]]]
+    #
     for keyName in settings.orcaModifierKeys:
         if keyName == "Caps_Lock":
             os.system('xmodmap -e "clear Lock"')
+        if keyName in ["Caps_Lock", "KP_Insert", "Insert"]:
+            command = 'xmodmap -e "keysym %s = %s"' % (keyName, keyName)
+            os.system(command)
 
     if _currentPresentationManager >= 0:
         _PRESENTATION_MANAGERS[_currentPresentationManager].activate()
@@ -908,6 +960,26 @@ def loadUserSettings(script=None, inputEvent=None):
 
     return True
 
+def _showAppPreferencesGUI(script=None, inputEvent=None):
+    """Displays the user interace to configure the settings for a
+    specific applications within Orca and set up those app-specific
+    user preferences using a GUI.
+
+    Returns True to indicate the input event has been consumed.
+    """
+
+    try:
+        module = __import__(settings.appGuiPreferencesModule,
+                            globals(),
+                            locals(),
+                            [''])
+        module.showPreferencesUI()
+    except:
+        debug.printException(debug.LEVEL_SEVERE)
+        pass
+
+    return True
+
 def _showPreferencesGUI(script=None, inputEvent=None):
     """Displays the user interace to configure Orca and set up
     user preferences using a GUI.
@@ -967,21 +1039,25 @@ def _showPreferencesConsole(script=None, inputEvent=None):
 
     return True
 
-def _showQuitGUI(script=None, inputEvent=None):
-    """Displays the user interace to quit Orca.
+def quitOrca(script=None, inputEvent=None):
+    """Quit Orca. Check if the user wants to confirm this action.
+    If so, show the confirmation GUI otherwise just shutdown.
 
     Returns True to indicate the input event has been consumed.
     """
 
-    try:
-        module = __import__(settings.quitModule,
-                            globals(),
-                            locals(),
-                            [''])
-        module.showQuitUI()
-    except:
-        debug.printException(debug.LEVEL_SEVERE)
-        pass
+    if settings.quitOrcaNoConfirmation:
+        shutdown()
+    else:
+        try:
+            module = __import__(settings.quitModule,
+                                globals(),
+                                locals(),
+                                [''])
+            module.showQuitUI()
+        except:
+            debug.printException(debug.LEVEL_SEVERE)
+            pass
 
     return True
 
@@ -1067,25 +1143,7 @@ def start(registry):
         signal.signal(signal.SIGALRM, settings.timeoutCallback)
         signal.alarm(settings.timeoutTime)
 
-    try:
-        speech.speak(_("Welcome to Orca."))
-        braille.displayMessage(_("Welcome to Orca."))
-    except:
-        debug.printException(debug.LEVEL_SEVERE)
-
     if not _PRESENTATION_MANAGERS:
-
-        # [[[WDW - comment out hierarchical_presenter for now.  It
-        # relies on the list of known applications, and we've disabled
-        # that due to a hang in a call to getChildAtIndex in
-        # util.getKnownApplications.]]]
-        #
-        #import focus_tracking_presenter
-        #import hierarchical_presenter
-        #_PRESENTATION_MANAGERS = \
-        #    [focus_tracking_presenter.FocusTrackingPresenter(),
-        #     hierarchical_presenter.HierarchicalPresenter()]
-
         import focus_tracking_presenter
         _PRESENTATION_MANAGERS = \
             [focus_tracking_presenter.FocusTrackingPresenter()]
@@ -1125,7 +1183,9 @@ def shutdown(script=None, inputEvent=None):
         signal.signal(signal.SIGALRM, settings.timeoutCallback)
         signal.alarm(settings.timeoutTime)
 
-    speech.speak(_("goodbye."))
+    # Translators: this is what Orca speaks and brailles when it quits.
+    #
+    speech.speak(_("Goodbye."))
     braille.displayMessage(_("Goodbye."))
 
     # Deregister our event listeners
@@ -1208,21 +1268,21 @@ def abortOnSignal(signum, frame):
 
 def usage():
     """Prints out usage information."""
-    print _("Usage: orca [OPTION...]")
+    print "Usage: orca [OPTION...]"
     print
-    print _("-?, --help                   Show this help message")
-    print _("-v, --version                %s") % platform.version
-    print _("-s, --setup, --gui-setup     Set up user preferences")
-    print _("-t, --text-setup             Set up user preferences (text version)")
-    print _("-n, --no-setup               Skip set up of user preferences")
-    print _("-u, --user-prefs-dir=dirname Use alternate directory for user preferences")
-    print _("-e, --enable=[speech|braille|braille-monitor|magnifier|main-window] Force use of option")
-    print _("-d, --disable=[speech|braille|braille-monitor|magnifier|main-window] Prevent use of option")
-    print _("-q, --quit                   Quits Orca (if shell script used)")
+    print "-?, --help                   Show this help message"
+    print "-v, --version                %s" % platform.version
+    print "-s, --setup, --gui-setup     Set up user preferences"
+    print "-t, --text-setup             Set up user preferences (text version)"
+    print "-n, --no-setup               Skip set up of user preferences"
+    print "-u, --user-prefs-dir=dirname Use alternate directory for user preferences"
+    print "-e, --enable=[speech|braille|braille-monitor|magnifier|main-window] Force use of option"
+    print "-d, --disable=[speech|braille|braille-monitor|magnifier|main-window] Prevent use of option"
+    print "-q, --quit                   Quits Orca (if shell script used)"
     print
-    print _("If Orca has not been previously set up by the user, Orca\nwill automatically launch the preferences set up unless\nthe -n or --no-setup option is used.")
+    print "If Orca has not been previously set up by the user, Orca\nwill automatically launch the preferences set up unless\nthe -n or --no-setup option is used."
     print
-    print _("Report bugs to orca-list@gnome.org.")
+    print "Report bugs to orca-list@gnome.org."
     pass
 
 def main():
@@ -1251,7 +1311,6 @@ def main():
     #
     desktopRunning = False
     try:
-        import gtk
         if gtk.gdk.display_get_default():
             desktopRunning = True
     except:
@@ -1367,10 +1426,8 @@ def main():
     # setting is typically not set since the gdm user does not
     # have a home.
     #
-    import commands
-    a11yEnabled = commands.getoutput(\
-        "gconftool-2 --get /desktop/gnome/interface/accessibility")
-    if (not bypassSetup) and (a11yEnabled != "true"):
+    a11yEnabled = settings.isAccessibilityEnabled()
+    if (not bypassSetup) and (not a11yEnabled):
         _showPreferencesConsole()
         abort()
 
@@ -1390,6 +1447,13 @@ def main():
     registry = atspi.Registry()
     init(registry)
 
+    try:
+        message = _("Welcome to Orca.")
+        speech.speak(message)
+        braille.displayMessage(message)
+    except:
+        debug.printException(debug.LEVEL_SEVERE)
+
     # Check to see if the user wants the configuration GUI. It's
     # done here so that the user's existing preferences can be used
     # to set the initial GUI state.  We'll also force the set to
