@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2004-2006 Sun Microsystems Inc.
+# Copyright 2004-2007 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -33,10 +33,10 @@ created in their own module.  The module defining the Script subclass
 is also required to have a 'getScript(app)' method that returns an
 instance of the Script subclass.  See default.py for an example."""
 
-__id__        = "$Id: script.py 2365 2007-05-09 20:43:34Z richb $"
-__version__   = "$Revision: 2365 $"
-__date__      = "$Date: 2007-05-09 16:43:34 -0400 (Wed, 09 May 2007) $"
-__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__id__        = "$Id: script.py 2463 2007-06-06 18:03:27Z wwalker $"
+__version__   = "$Revision: 2463 $"
+__date__      = "$Date: 2007-06-06 14:03:27 -0400 (Wed, 06 Jun 2007) $"
+__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import braillegenerator
@@ -135,7 +135,7 @@ class Script:
         matches = keybindings.KeyBindings()
 
         for binding in self.keyBindings.keyBindings:
-            if inputEventHandler._function == binding.handler._function:
+            if inputEventHandler == binding.handler:
                 matches.add(binding)
 
         return matches
@@ -158,13 +158,9 @@ class Script:
         Returns a list (possibly empty) of BrlTTY commands (they're in
         braille.py) that match the given inputEventHandler passed.
         """
-        matches = []
-
-        for command,handler in self.brailleBindings.iteritems():
-            if inputEventHandler._function == handler._function:
-                matches.append(command)
-
-        return matches
+        return [command
+                for command, handler in self.brailleBindings.iteritems()
+                if inputEventHandler == handler]
 
     def getBrailleGenerator(self):
         """Returns the braille generator for this script.
@@ -206,6 +202,20 @@ class Script:
 
         return keyBindings
 
+    def getAppState(self):
+        """Returns an object that can be passed to setAppState.  This
+        object will be used by setAppState to restore any state
+        information that was being maintained by the script."""
+        return None
+
+    def setAppState(self, appState):
+        """Sets the application state using the given appState object.
+
+        Arguments:
+        - appState: an object obtained from getAppState
+        """
+        return
+        
     # [[[WDW - There is a circular reference going on somewhere (see
     # bug 333168).  In the presence of this reference, the existence
     # of a __del__ method prevents the garbage collector from
