@@ -19,9 +19,9 @@
 
 """Custom script for Evolution."""
 
-__id__        = "$Id: Evolution.py 2375 2007-05-11 17:56:08Z richb $"
-__version__   = "$Revision: 2375 $"
-__date__      = "$Date: 2007-05-11 13:56:08 -0400 (Fri, 11 May 2007) $"
+__id__        = "$Id: Evolution.py 2420 2007-05-23 22:10:42Z richb $"
+__version__   = "$Revision: 2420 $"
+__date__      = "$Date: 2007-05-23 18:10:42 -0400 (Wed, 23 May 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -84,6 +84,133 @@ class WhereAmI(where_am_I.WhereAmI):
 
         return text
 
+    def _hasTextSelections(self, obj):
+        """Return an indication of whether this object has selected text.
+        Note that it's possible that this object has no text, but is part
+        of a selected text area. Because of this, we need to check the
+        objects on either side to see if they are none zero length and 
+        have text selections.
+
+        Arguments:
+        - obj: the text object to start checking for selected text.
+
+        Returns: an indication of whether this object has selected text,
+        or adjacent text objects have selected text.
+        """
+
+        currentSelected = False
+        otherSelected = False
+        nSelections = obj.text.getNSelections()
+        if nSelections:
+            currentSelected = True
+        else:
+            otherSelected = False
+            displayedText = obj.text.getText(0, -1)
+            if len(displayedText) == 0:
+                container = obj.parent.parent
+                current = obj.parent.index
+                morePossibleSelections = True
+                while morePossibleSelections:
+                    morePossibleSelections = False
+                    if (current-1) >= 0:
+                        prevPanel = container.child(current-1)
+                        prevObj = prevPanel.child(0)
+                        if prevObj.text.getNSelections() > 0:
+                            otherSelected = True
+                        else:
+                            displayedText = prevObj.text.getText(0, -1)
+                            if len(displayedText) == 0:
+                                current -= 1
+                                morePossibleSelections = True
+
+                current = obj.parent.index
+                morePossibleSelections = True
+                while morePossibleSelections:
+                    morePossibleSelections = False
+                    if (current+1) < container.childCount:
+                        nextPanel = container.child(current+1)
+                        nextObj = nextPanel.child(0)
+                        if nextObj.text.getNSelections() > 0:
+                            otherSelected = True
+                        else:
+                            displayedText = nextObj.text.getText(0, -1)
+                            if len(displayedText) == 0:
+                                current += 1
+                                morePossibleSelections = True
+
+        return [currentSelected, otherSelected]
+
+    def _getTextSelections(self, obj, doubleClick):
+        """Get all the text applicable text selections for the given object.
+        If the user doubleclicked, look to see if there are any previous
+        or next text objects that also have selected text and add in their
+        text contents.
+
+        Arguments:
+        - obj: the text object to start extracting the selected text from.
+        - doubleClick: True if the user double-clicked the "where am I" key.
+
+        Returns: all the selected text contents plus the start and end
+        offsets within the text for the given object.
+        """
+
+        textContents = ""
+        startOffset = 0
+        endOffset = 0
+        if obj.text.getNSelections() > 0:
+            [textContents, startOffset, endOffset] = \
+                                            self._getTextSelection(obj)
+
+        if doubleClick:
+            # Unfortunately, Evolution doesn't use the FLOWS_FROM and 
+            # FLOWS_TO relationships to easily allow us to get to previous 
+            # and next text objects. Instead we have to move up the
+            # component hierarchy until we get to the object containing all
+            # the panels (with each line containing a single text item).
+            # We can then check in both directions to see if there is other
+            # contiguous text that is selected. We also have to jump over 
+            # zero length (empty) text lines and continue checking on the 
+            # other side.
+            #
+            container = obj.parent.parent
+            current = obj.parent.index
+            morePossibleSelections = True
+            while morePossibleSelections:
+                morePossibleSelections = False
+                if (current-1) >= 0:
+                    prevPanel = container.child(current-1)
+                    prevObj = prevPanel.child(0)
+                    displayedText = prevObj.text.getText(0, -1)
+                    if len(displayedText) == 0:
+                        current -= 1
+                        morePossibleSelections = True
+                    elif prevObj.text.getNSelections() > 0:
+                        [newTextContents, start, end] = \
+                                     self._getTextSelection(prevObj)
+                        textContents = newTextContents + " " + textContents
+                        current -= 1
+                        morePossibleSelections = True
+
+            current = obj.parent.index
+            morePossibleSelections = True
+            while morePossibleSelections:
+                morePossibleSelections = False
+                if (current+1) < container.childCount:
+                    nextPanel = container.child(current+1)
+                    nextObj = nextPanel.child(0)
+                    displayedText = nextObj.text.getText(0, -1)
+                    if len(displayedText) == 0:
+                        current += 1
+                        morePossibleSelections = True
+                    elif nextObj.text.getNSelections() > 0:
+                        [newTextContents, start, end] = \
+                                     self._getTextSelection(nextObj)
+                        textContents += " " + newTextContents
+                        current += 1
+                        morePossibleSelections = True
+
+        return [textContents, startOffset, endOffset]
+
 ########################################################################
 #                                                                      #
 # The Evolution script class.                                          #
