@@ -21,9 +21,9 @@
 fallback to local settings if the user settings doesn't exist (e.g., in the
 case of gdm) or doesn't have the specified attribute."""
 
-__id__        = "$Id: settings.py 2434 2007-05-29 15:18:37Z richb $"
-__version__   = "$Revision: 2434 $"
-__date__      = "$Date: 2007-05-29 11:18:37 -0400 (Tue, 29 May 2007) $"
+__id__        = "$Id: settings.py 2469 2007-06-12 20:19:09Z richb $"
+__version__   = "$Revision: 2469 $"
+__date__      = "$Date: 2007-06-12 16:19:09 -0400 (Tue, 12 Jun 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -129,6 +129,7 @@ userCustomizableSettings = [
     "sayAllStyle",
     "keyboardLayout",
     "speakBlankLines",
+    "enabledTextAttributes",
 ]
 
 # The name of the module that hold the user interface for the main window
@@ -397,6 +398,10 @@ speakBlankLines         = True
 #
 readTableCellRow    = True
 
+# The complete list of possible text attributes.
+#
+allTextAttributes = "bg-color:; bg-full-height:; bg-stipple:; direction:; editable:; family-name:; fg-color:; fg-stipple:; font-effect:none; indent:0; invisible:; justification:left; language:; left-margin:; line-height:100%; paragraph-style:Default; pixels-above-lines:; pixels-below-lines:; pixels-inside-wrap:; right-margin:; rise:; scale:; size:; stretch:; strikethrough:false; style:normal; text-decoration:none; text-rotation:0; text-shadow:none; underline:none; variant:; vertical-align:baseline; weight:400; wrap-mode:; writing-mode:lr-tb;"
+
 # The default set of text attributes to speak to the user. Specific
 # application scripts (or individual users can override these values if
 # so desired. Each of these text attributes is of the form <key>:<value>;
@@ -479,6 +484,12 @@ useBonoboMain           = True
 #
 debugEventQueue         = False
 
+# If True, we collect information regarding memory usage and provide
+# keystrokes to dump the usage information to the console:
+# Orca+Ctrl+F8 for brief, Orca+Shift+Ctrl+F8 for detailed.
+#
+debugMemoryUsage        = False
+
 # The timeout value (in seconds) and callback used to determine if
 # Orca has hung or not.  The only setting one should muck with here is
 # the timeoutTime unless you want to create a custom callback handler
